#include <stdlib.h>
#include <imsls.h>

float get_ss(int n, int n_independent, int *n_levels, float *x);

#define N 24
#define N_INDEPENDENT 3

int main()
{
  char *col_labels[] = {" ", "Whole", "Split", "Block", " "};
  int i;
  int n = N;
  int n_independent = N_INDEPENDENT;  
  int whole[N]={1,1,1,1,1,1,
	       2,2,2,2,2,2,
	       3,3,3,3,3,3,
	       4,4,4,4,4,4};
  int split[N]={1,2,3,1,2,3,
	       1,2,3,1,2,3,
	       1,2,3,1,2,3,
	       1,2,3,1,2,3};
  int block[N]={1,1,1,2,2,2,
	       1,1,1,2,2,2,
	       1,1,1,2,2,2,
	       1,1,1,2,2,2};
  float y[N] ={0.0,  53.8, 49.5, 41.6, 0.0,  53.8,
               53.3, 57.6, 59.8, 69.6, 69.6, 65.8,
	       62.3, 63.4, 64.5, 58.5, 50.4, 46.1, 
	       75.4, 70.3, 68.8, 65.6, 67.3, 65.3};
  
  float x[N][N_INDEPENDENT+1];
  float error_ss;
  int *missing_idx;
  int n_missing;

  /* Set the first and fifth observations to missing values. */
  y[0] = imsls_f_machine(6);
  y[4] = imsls_f_machine(6);

  /* Fill the array x with the classification variables and observations. */
  for (i=0;i<n; i++) {
    x[i][0] = (float)whole[i]; 
    x[i][1] = (float)split[i]; 
    x[i][2] = (float)block[i]; 
    x[i][3] = y[i];
  }
  /* Sort the data since imsls_f_anova_factorial expects sorted data. */
  imsls_f_sort_data(n, n_independent+1, (float*)x, 3, 0);
  
  n_missing = imsls_f_yates(n, n_independent, (float *)&(x[0][0]),
			    IMSLS_DESIGN, 2, 
			    IMSLS_GET_SS, get_ss,
			    IMSLS_ERROR_SS, &error_ss,
			    IMSLS_MISSING_INDEX, &missing_idx,
			    0);
  printf("Returned error sum of squares = %f\n\n", error_ss);
  printf("Missing values replaced: %d\n", n_missing);
  printf("Whole     Split    Block    Estimate\n");
  for (i=0;i<n_missing;i++) {
    printf("%3d        %3d      %3d      %7.3f\n", 
	   (int)x[missing_idx[i]][0],
	   (int)x[missing_idx[i]][1],
	   (int)x[missing_idx[i]][2],
	   x[missing_idx[i]][n_independent]);
  }
  imsls_f_write_matrix("Sorted x, with estimates", n, n_independent+1,
		            (float*)x, 
                       IMSLS_WRITE_FORMAT, "%-4.0f%-4.0f%-4.0f%5.2f", 
		           IMSLS_COL_LABELS, col_labels, 
		           IMSLS_NO_ROW_LABELS, 0);
  
}

float get_ss(int n, int n_independent, int *n_levels, float *x)
{
  int i;
  float errorSS, pValue;
  float *test_effects = NULL;
  float *anova_table = NULL;
  float responses[24];
  /* 
   * Copy responses from the last column of x into a 1-D array 
   * as expected by imsls_f_anova_factorial. 
   */
  for (i=0;i<n;i++) {
    responses[i] = x[i*(n_independent+1)+n_independent];
  }
  /*
   * Compute the error sum of squares.
   */
  pValue = imsls_f_anova_factorial(n_independent, n_levels, responses,
				   IMSLS_TEST_EFFECTS, &test_effects,
				   IMSLS_ANOVA_TABLE, &anova_table,
				   IMSLS_POOL_INTERACTIONS, 0);
  errorSS = anova_table[4] + test_effects[21]; 

  /* Free memory returned by imsls_f_anova_factorial. */
  if (test_effects != NULL) imsls_free(test_effects);
  if (anova_table != NULL) imsls_free(anova_table);

  return errorSS;
}

After running this code to replace missing values with Yates estimates, it would be followed by a call to the split-plot analysis of variance:

float *aov_table, y[24];
int expunit[24], whole[24], split[24];
for(int i=0; i < 24; i++){whole[i]  = x[i];    split[i] = x[i+24]; 
                          expunit[i]= x[i+48]; y[i]     = x[i+72];}
float aov_table = imsls_f_split_plot (24, 1, 4, 3, expunit, whole, 
                                     split, y[], 0);
